searchData={"items":[{"type":"module","title":"Whisperer","doc":"Documentation for `Whisperer`.","ref":"Whisperer.html"},{"type":"function","title":"Whisperer.add_agent/2","doc":"Adds an agent to the orchestrator for the given session.\nAn agent is a module that implements the `Whisperer.Agent` behaviour.\n\nIt returns an `:ok` tuple.","ref":"Whisperer.html#add_agent/2"},{"type":"function","title":"Examples - Whisperer.add_agent/2","doc":"{:ok, _pid} = Whisperer.start_session(\"123\", MyApp.Sequencer.Basic, %{})\n    Whisperer.add_agent(\"123\", MyApp.Agent)\n    :ok","ref":"Whisperer.html#add_agent/2-examples"},{"type":"function","title":"Whisperer.get_conversation/1","doc":"Gets the conversation for a session.\n\nIt returns the conversation history for the session or an `:error` tuple if there was an issue getting the conversation.","ref":"Whisperer.html#get_conversation/1"},{"type":"function","title":"Examples - Whisperer.get_conversation/1","doc":"Whisperer.get_conversation(\"123\")\n    [%Whisperer.Message{content: \"Hello, how are you?\"}, %Whisperer.Message{content: \"I'm doing great, thank you!\"}]","ref":"Whisperer.html#get_conversation/1-examples"},{"type":"function","title":"Whisperer.process_user_input/2","doc":"Processes user input for a session.\n\nIt returns an `:ok` tuple with the processed message or an `:error` tuple if there was an issue processing the message.\n\nInternally, it uses the Sequencer module provided when starting the session to create a sequence of agents to be used to process the input, and then uses the Orchestrator to process the message through the sequence.","ref":"Whisperer.html#process_user_input/2"},{"type":"function","title":"Examples - Whisperer.process_user_input/2","doc":"Whisperer.process_user_input(\"123\", \"Hello, how are you?\")\n    {:ok, %Whisperer.Message{content: \"I'm doing great, thank you!\"}}","ref":"Whisperer.html#process_user_input/2-examples"},{"type":"function","title":"Whisperer.start_session/3","doc":"Starts a new session to be used with the orchestrator.\nThis can be directly mapped to a websocket connection on your application for a user session.\n\nIt accepts the following arguments:\n- an identifier for the session (e.g. a websocket connection id or even a name; just has to be a unique identifier)\n- a sequencer module. This is the module defined in your application that will handle the sequencing of the agents. It must implement the `Whisperer.Sequencer` behaviour.\n- a context map. This is a map of context that will be passed to the agents and used to process the messages.\nYou can use this to pass in any information you want to be available to the agents, e.g user information, session information, observability information, etc.\n\nIt returns an `:ok` tuple with the pid of the orchestrator process.","ref":"Whisperer.html#start_session/3"},{"type":"function","title":"Examples - Whisperer.start_session/3","doc":"{:ok, _pid} = Whisperer.start_session(\"123\", MyApp.Sequencer.Basic, %{})","ref":"Whisperer.html#start_session/3-examples"},{"type":"module","title":"Whisperer.Message","doc":"Defines the message structure used by the orchestrator","ref":"Whisperer.Message.html"},{"type":"type","title":"Whisperer.Message.content/0","doc":"","ref":"Whisperer.Message.html#t:content/0"},{"type":"type","title":"Whisperer.Message.t/0","doc":"","ref":"Whisperer.Message.html#t:t/0"},{"type":"module","title":"Whisperer.Orchestrator.State","doc":"Defines the state used by the orchestrator","ref":"Whisperer.Orchestrator.State.html"},{"type":"type","title":"Whisperer.Orchestrator.State.context/0","doc":"","ref":"Whisperer.Orchestrator.State.html#t:context/0"},{"type":"type","title":"Whisperer.Orchestrator.State.conversations/0","doc":"","ref":"Whisperer.Orchestrator.State.html#t:conversations/0"},{"type":"type","title":"Whisperer.Orchestrator.State.t/0","doc":"","ref":"Whisperer.Orchestrator.State.html#t:t/0"},{"type":"module","title":"Whisperer.Orchestrator.Supervisor","doc":"Dynamic supervisor for orchestrator processes.\nEach session gets its own orchestrator process.","ref":"Whisperer.Orchestrator.Supervisor.html"},{"type":"function","title":"Whisperer.Orchestrator.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Whisperer.Orchestrator.Supervisor.html#child_spec/1"},{"type":"function","title":"Whisperer.Orchestrator.Supervisor.get_orchestrator/1","doc":"Gets the orchestrator process for a session, starting one if it doesn't exist.","ref":"Whisperer.Orchestrator.Supervisor.html#get_orchestrator/1"},{"type":"function","title":"Whisperer.Orchestrator.Supervisor.start_link/1","doc":"","ref":"Whisperer.Orchestrator.Supervisor.html#start_link/1"},{"type":"function","title":"Whisperer.Orchestrator.Supervisor.start_orchestrator/3","doc":"Starts a new orchestrator process for a session.","ref":"Whisperer.Orchestrator.Supervisor.html#start_orchestrator/3"},{"type":"function","title":"Whisperer.Orchestrator.Supervisor.via_tuple/1","doc":"Returns the process name for a session's orchestrator.","ref":"Whisperer.Orchestrator.Supervisor.html#via_tuple/1"},{"type":"behaviour","title":"Whisperer.Agent","doc":"This behaviour defines the interface for agents in the Whisperer system.\n\nAgents are the building blocks of the Whisperer system. They are responsible for processing messages and generating responses.\nThe Agent can be an LLM, a simple function, or a combination of both. It can be any kind of program that can take an input, perform some kind of task and return a response.\n\nThis behaviour expects the implementing module to export a `characteristics/0` function that returns a map containing the agent's id, name, description, and capabilities.\nIt also expects the implementing module to export a `process_message/3` function that takes in a message, the current context(a map of key-value pairs that can be used to store any information needed by the agent), and the conversation history, and returns an `:ok` tuple with the processed message or an `:error` tuple if there was an issue processing the message.","ref":"Whisperer.Agent.html"},{"type":"behaviour","title":"Examples - Whisperer.Agent","doc":"iex> MyApp.Agent.characteristics()\n    %{id: \"1\", name: \"Agent_a\", description: \"Agent a\", capabilities: [\"capability1\"]}\n\n    iex> MyApp.Agent.process_message(%Message{content: \"Hello, how are you?\"}, %{trace_id: \"123\"}, [])\n    {:ok, %Message{content: \"I'm doing great, thank you!\"}}","ref":"Whisperer.Agent.html#module-examples"},{"type":"callback","title":"Whisperer.Agent.characteristics/0","doc":"","ref":"Whisperer.Agent.html#c:characteristics/0"},{"type":"callback","title":"Whisperer.Agent.process_message/3","doc":"","ref":"Whisperer.Agent.html#c:process_message/3"},{"type":"type","title":"Whisperer.Agent.agent_characteristics/0","doc":"","ref":"Whisperer.Agent.html#t:agent_characteristics/0"},{"type":"type","title":"Whisperer.Agent.agent_description/0","doc":"","ref":"Whisperer.Agent.html#t:agent_description/0"},{"type":"type","title":"Whisperer.Agent.agent_id/0","doc":"","ref":"Whisperer.Agent.html#t:agent_id/0"},{"type":"type","title":"Whisperer.Agent.agent_name/0","doc":"","ref":"Whisperer.Agent.html#t:agent_name/0"},{"type":"type","title":"Whisperer.Agent.context/0","doc":"","ref":"Whisperer.Agent.html#t:context/0"},{"type":"type","title":"Whisperer.Agent.conversation_history/0","doc":"","ref":"Whisperer.Agent.html#t:conversation_history/0"},{"type":"module","title":"Whisperer.Sequence","doc":"Defines the relationship between the agents as an execution sequence.","ref":"Whisperer.Sequence.html"},{"type":"type","title":"Whisperer.Sequence.connections/0","doc":"","ref":"Whisperer.Sequence.html#t:connections/0"},{"type":"type","title":"Whisperer.Sequence.t/0","doc":"","ref":"Whisperer.Sequence.html#t:t/0"},{"type":"behaviour","title":"Whisperer.Sequencer","doc":"This behaviour defines the interface for a sequencer in the Whisperer system.\n\nA sequencer is a module responsible for selecting the agents to be used based on the user input, and generating a sequence of agents that will be used to process the input.\nThe sequencer does not handle the actual processing of messages, it only decides which agents from the given list of agents should be used and in what order they should be used by the orchestrator.\n\nThis behaviour expects the implementing module to export a `create_sequence/3` function that takes in the user input, the list of all available agents in a specified format, and the conversation history (for context window purposes), and returns an `:ok` tuple with the sequence or an `:error` tuple if there was an issue creating the sequence.\n\nThe `create_sequence/3` function should return a `Whisperer.Sequence` struct that contains the starting agent and a graph that shows the connections between the agents.\n\nThe implementation of the `create_sequence/3` function should take into account the input, capabilities of the agents and the context window of the conversation history. The implementation can use an LLM or just a simple algorithm to generate the sequence using regular functions.","ref":"Whisperer.Sequencer.html"},{"type":"behaviour","title":"Examples - Whisperer.Sequencer","doc":"iex> MyApp.Sequencer.create_sequence(\"Hello, how are you?\", [%{id: \"1\", name: \"Agent 1\", description: \"Agent 1\", capabilities: [\"capability1\"]}, %{id: \"2\", name: \"Agent 2\", description: \"Agent 2\", capabilities: [\"capability2\"]}], [])\n    {:ok, %Whisperer.Sequence{\n      start_agent: \"agent_a\",\n      connections: %{\"agent_a\" => [\"agent_b\"], \"agent_b\" => [\"agent_c\"]}\n    }}","ref":"Whisperer.Sequencer.html#module-examples"},{"type":"callback","title":"Whisperer.Sequencer.create_sequence/3","doc":"","ref":"Whisperer.Sequencer.html#c:create_sequence/3"},{"type":"module","title":"Whisperer.Orchestrator","doc":"Main orchestrator module that manages the flow of messages between users and agents.\nMaintains state in its own process using GenServer.\nTODO: Explore conversation history per agent","ref":"Whisperer.Orchestrator.html"},{"type":"function","title":"Whisperer.Orchestrator.add_agent/2","doc":"Registers a new agent with the orchestrator.","ref":"Whisperer.Orchestrator.html#add_agent/2"},{"type":"function","title":"Whisperer.Orchestrator.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Whisperer.Orchestrator.html#child_spec/1"},{"type":"function","title":"Whisperer.Orchestrator.get_conversation/1","doc":"Gets the conversation history for a user's session.","ref":"Whisperer.Orchestrator.html#get_conversation/1"},{"type":"function","title":"Whisperer.Orchestrator.process_user_input/2","doc":"Processes a user message through the appropriate agent.\n1. Adds the received user message to the conversation history.\n2. Uses the sequencer to create a plan for executing the agents.","ref":"Whisperer.Orchestrator.html#process_user_input/2"},{"type":"function","title":"Whisperer.Orchestrator.start_link/1","doc":"Starts the orchestrator process.","ref":"Whisperer.Orchestrator.html#start_link/1"},{"type":"extras","title":"Whisperer","doc":"# Whisperer\n\n[![Actions Status](https://github.com/monitor-lizzard/whisperer/workflows/CI/badge.svg)](https://github.com/Monitor-Lizzard/whisperer/actions?query=workflow:CI) [![Hex.pm](https://img.shields.io/hexpm/v/whisperer.svg)](https://hex.pm/packages/whisperer) [![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/whisperer)\n\n\n**Whisperer is an unopionated multi-agent framework in Elixir**","ref":"readme.html"},{"type":"extras","title":"Features - Whisperer","doc":"- **No-batteries-included**: With Whisperer, you can build your own agents with any implementations and logic you want\n- **Resilient orchestration**: Whisperer orchestrates and manages the lifecycle of your agents using battle-tested OTP strategies\n- **Scalable**: Whisperer is built on top of Elixir's OTP, which means it can scale to millions of agents with ease\n- **Fault-tolerant**: Whisperer is built to be fault-tolerant, meaning that it can recover from failures and continue to operate","ref":"readme.html#features"},{"type":"extras","title":"Workflow Structure - Whisperer","doc":"![Workflow Struture](https://raw.githubusercontent.com/Monitor-Lizzard/whisperer/main/img/flow.png)","ref":"readme.html#workflow-structure"},{"type":"extras","title":"Key Components - Whisperer","doc":"","ref":"readme.html#key-components"},{"type":"extras","title":"Orchestrator - Whisperer","doc":"The Orchestrator is the main component of Whisperer. It is responsible for managing the lifecycle of agents and orchestrating the communication between them. The Orchestrator is the entry point for all agents and is responsible for starting, stopping, and monitoring agents.","ref":"readme.html#orchestrator"},{"type":"extras","title":"Agent - Whisperer","doc":"An Agent is a worker that performs a specific task. Agents can be implemented in any way you want, as long as they implement the `Whisperer.Agent` behaviour. Agents can communicate with each other using messages.","ref":"readme.html#agent"},{"type":"extras","title":"Message - Whisperer","doc":"A Message is a data structure that is sent between agents. Messages can contain any data you want, as long as it is serializable. Messages are used to communicate between agents and the Orchestrator.","ref":"readme.html#message"},{"type":"extras","title":"Sequence - Whisperer","doc":"A Sequence is a collection of agents that are executed in a specific order. Sequences can be used to model complex workflows and orchestrate the work of multiple agents.\n\nIf [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `whisperer` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:whisperer, \"~> 0.1.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#sequence"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.1.0 (2024-12-05) - Changelog","doc":"","ref":"changelog.html#0-1-0-2024-12-05"},{"type":"extras","title":"Features - Changelog","doc":"- Provide unidirectional template for relationship with Agents\n- Add Support for `Message`, `Agent`, `Workflow` and `Orchestrator`","ref":"changelog.html#features"},{"type":"extras","title":"License","doc":"# MIT License\n\nCopyright (c) 2024 Abstract Machine Labs, Inc.\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}